#-----------------------------------------------------------------------------
# Loader
# Copyright (c) 2003 David G. Drumright (ockham)
# 10/17/03
# 11/02/03 Added sorting, fixed bug in Protected Load, fixed
# extra slash, found that mirroring wasn't happening.
#-----------------------------------------------------------------------------
import os
import glob
import string
import shutil
from Tkinter import *

import tkMessageBox

import poser
scene=poser.Scene()

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

global IniName
global BasePath
global Patterns
global DefaultDirs
global LatestDirs
global FileList
global PossDrives
global StuffLabel
global DirDisplayList
global DirFullList
global BaseFig
global SearchList

#-----------------------------------------------------------------------------

BasePathUncut=poser.AppLocation()  # So we can strip off the full paths given by tkFileDialog
BasePath=BasePathUncut[:(-len('poser.exe'))]
IniName=os.path.normpath(BasePath+'Loader.ini')
PossDrives=['A:/'] # Assume A is here to begin with. Reminder: This means that A is index 0.
BaseFig=0

# Set of file-groups for each main choice.  Defining strings for clarity of source.
NOPs='*'
CR2s='*.CR2 *.CRZ'
PZ2s='*.PZ2 *.P2Z'
FC2s='*.FC2 *.FCZ'
HR2s='*.HR2 *.HRZ *.CR2 *.CRZ *.PP2 *.P2Z'
HD2s='*.HD2 *.HDZ'
PP2s='*.PP2 *.P2Z *.CR2 *.CRZ'
LT2s='*.LT2 *.LTZ'
CM2s='*.CM2 *.CMZ'
OBJs='*.OBJ *.3DS *.BVH *.DXF *.3DM*'
MORs='*.OBJ'
ETCs='*.PZ3 *.PZZ *.PP2 *.PPZ *.PZ2 *.P2Z *.CR2 *.CRZ *.FC2 *.FCZ *.HD2 *.HDZ *.LT2 *.LTZ *.CM2 *.CMZ'

Patterns=[
NOPs, #  0 no choice
CR2s, #  1 Figure (new)
CR2s, #  2 Figure (conformed)
PZ2s, #  3 Pose (default)
PZ2s, #  4 Pose (mirror)
PZ2s, #  5 Mat Pose (1)
PZ2s, #  6 Mat Pose (2)
PZ2s, #  7 Mor Pose (1)
PZ2s, #  8 Mor Pose (2)
FC2s, #  9 Face (default)
FC2s, # 10 Face (mirror)
HR2s, # 11 Hair (default)
HR2s, # 12 Hair (parented)
HD2s, # 13 Hand (left)
HD2s, # 14 Hand (right)
PP2s, # 15 Prop (single)
PP2s, # 16 Prop (parented)
LT2s, # 17 Light set (default)
LT2s, # 18 Light set (parented to current selection)
CM2s, # 19 Camera set (default)
CM2s, # 20 Camera set (parented to current selection)
OBJs, # 21 Mesh (chose the most common and most supported ones.)
MORs, # 22 Morph
ETCs, # 23 Whatever (default)
ETCs, # 24 Whatever (parented)
]

# Initial directories, to be tacked onto the basic AppLocation before use.
# Define the strings, mainly to separate versions on Primitives
RB='Runtime/'
LB=RB+'Libraries/'
CB=LB+'character/'
PP=CB+'People/'
PS=LB+'Pose/Pose Sets/'
CF=CB+'Clothing-Female/'
CM=CB+'Clothing-Male/'
if float(poser.Version()) >= 5.0:
    PT=LB+'props/Primitives/'
else:
    PT=LB+'props/Prop Types/'
BE=LB+'face/Basic Expressions/'
HT=LB+'hair/Hair Types/'
BH=LB+'hand/Basic Hands/'
LS=LB+'light/Light Sets/'
CS=LB+'camera/Camera Sets/'
OB=RB+'Geometries/props/'
MT=LB+'morphs/'

DefaultDirs=[
RB,   #  0 no choice
PP,   #  1 Figure (new)
CF,   #  2 Figure (replace)
PS,   #  3 Pose (default)
PS,   #  4 Pose (mirror)
PS,   #  5 Mat Pose (1)
PS,   #  6 Mat Pose (2)
PS,   #  7 Mor Pose (1)
PS,   #  8 Mor Pose (2)
BE,   #  9 Face (default)
BE,   # 10 Face (mirror)
HT,   # 11 Hair (default)
HT,   # 12 Hair (parented)
BH,   # 13 Hand (left)
BH,   # 14 Hand (right)
PT,   # 15 Prop (default)
PT,   # 16 Prop (parented)
LS,   # 17 Light set (default)
LS,   # 18 Light set (parented)
CS,   # 19 Camera set (default)
CS,   # 20 Camera set (parented)
OB,   # 21 Import mesh
MT,   # 22 Import morph
CF,   # 23 Stuff (default)
CF,   # 24 Stuff (parented)
]


LatestDirs=[]
FileList=[]  # Begin with empty
StuffLabel='STUFF'
DirDisplayList=[]
DirFullList=[]
SearchList={}

#-----------------------------------------------------------------------------
def lccompare(a,b):
    return cmp(string.lower(a), string.lower(b))
#-----------------------------------------------------------------------------
def FindOBJ(Matcher,dirname,names):  # helper for ProtectedLoad. Needs to be outside the app
    global SearchList
    UpPat=string.upper(Matcher)
    for OneBaseName in names:
        if string.count(string.upper(OneBaseName),UpPat):
            SearchList[OneBaseName]=dirname

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global StuffLabel
        self.master=master
        master.title('Loader')
        self.Quantity=1
        self.Choice=0 # Will be revised by StartIni
        self.StartIni()  # Initialize the INI file, and fill in the LatestDirs.

        # Radiobuttons.
        self.FileTypeVar=IntVar()
        self.FileTypeVar.set(self.Choice) # Choice was set by StartIni

        # Large frame to left, for the type buttons.
        self.ButtonFrame=Frame(self.master,bg='white')
        self.ButtonFrame.grid(row=1,column=0,padx=5)
        # Large frame to right, for the file select lists.
        self.FileFrame=Frame(self.master,bg='white')
        self.FileFrame.grid(row=1,column=1,padx=5)

        # The choice dots
        Titles1=['FIGURE:', 'POSE:', 'MAT POSE:','MOR POSE:','FACE:','HAIR:', 'HAND:', 'PROP:', 'LIGHT:', 'CAMERA:', 'IMPORT:']

        Titles2=['default', 'default', '1', '1', 'default', 'default','left', 'default', 'default', 'default', 'mesh']

        Titles3=['conformed','mirror', '2', '2', 'mirror', 'parented','right', 'parented', 'parented', 'parented','morph']

        R=0
        V=1
        self.Buttons={}
        for R in range(11):
            Label(self.ButtonFrame, text=Titles1[R],bg='white').grid(row=R, column=0,sticky=W)
            Label(self.ButtonFrame, text=Titles2[R],bg='white').grid(row=R, column=1,sticky=E)
            self.Buttons[V]=Radiobutton(self.ButtonFrame,bg='white',variable=self.FileTypeVar,value=V)
            self.Buttons[V].grid(row=R, column=2)
            V=V+1
            Label(self.ButtonFrame, text=Titles3[R],bg='white').grid(row=R, column=3,sticky=E)
            self.Buttons[V]=Radiobutton(self.ButtonFrame,bg='white',variable=self.FileTypeVar,value=V)
            self.Buttons[V].grid(row=R, column=4)
            V=V+1

        # The last has entry blank instead of fixed title.
        self.What1Entry=Entry(self.ButtonFrame,bg='white',width=10)
        self.What1Entry.grid(row=12, column=0)
        Label(self.ButtonFrame, text='default',bg='white').grid(row=12, column=1)
        self.Buttons[V]=Radiobutton(self.ButtonFrame,bg='white',variable=self.FileTypeVar,value=V)
        self.Buttons[V].grid(row=12, column=2)
        V=V+1
        Label(self.ButtonFrame, text='parented',bg='white').grid(row=12, column=3)
        self.Buttons[V]=Radiobutton(self.ButtonFrame,bg='white',variable=self.FileTypeVar,value=V)
        self.Buttons[V].grid(row=12, column=4)
        self.What1Entry.insert(0,StuffLabel)
        V=V+1

        # Inside the File Frame:
        # The quantity blank on top of files
        Label(self.FileFrame, text='Quantity:',bg='white').grid(row=0, column=2,sticky=E)
        self.QuantityEntry=Entry(self.FileFrame,bg='white',width=4)
        self.QuantityEntry.grid(row=0, column=3)
        self.QuantityEntry.delete(0,'end')
        self.QuantityEntry.insert(0,'1')

        # Dir list on left, File list on right.
        self.DirListYScroll=Scrollbar(self.FileFrame, orient=VERTICAL)
        self.DirListYScroll.grid( row=1, column=0,sticky=N+S+E)
        self.DirListXScroll=Scrollbar(self.FileFrame, orient=HORIZONTAL)
        self.DirListXScroll.grid( row=2, column=1)
        self.DirListBox=Listbox(self.FileFrame, height=15, selectmode=SINGLE,
                                  yscrollcommand=self.DirListYScroll.set,
                                  xscrollcommand=self.DirListXScroll.set,exportselection=0)
        self.DirListBox.grid( row=1, column=1)
        # Set the scrollbar to work
        self.DirListXScroll['command']=self.DirListBox.xview
        self.DirListYScroll['command']=self.DirListBox.yview

        self.FileListYScroll=Scrollbar(self.FileFrame, orient=VERTICAL)
        self.FileListYScroll.grid( row=1, column=3,sticky=N+S+E)
        self.FileListBox=Listbox(self.FileFrame, height=15, selectmode=MULTIPLE,
                                   yscrollcommand=self.FileListYScroll.set,exportselection=0)
        self.FileListBox.grid( row=1, column=2)
        self.FileListYScroll['command']=self.FileListBox.yview

        self.UndoButton=Button(self.FileFrame,text='Undo', command=self.HandleUndo)
        self.UndoButton.grid(row=0, column=1)
        self.UndoButton.configure(state=DISABLED)
        self.GoButton=Button(self.FileFrame,text='Go!', bg='pink',command=self.HandleGo)
        self.GoButton.grid(row=2, column=2,pady=2)

        # Do bindings last so all relevant controls have been defined.
        # Bind all the dots to HandleDots.
        for V in self.Buttons.keys():
            self.Buttons[V].bind('<Button-1>',self.HandleDots)
        # Use double-click in the folder list but not in the file list.
        self.DirListBox.bind('<Double-Button-1>',self.HandleSelectDir)

        # If you left a preference in the INI file, go ahead and form
        # up the appropriate dir by forcing the dot:
        self.FileTypeVar.set(self.Choice) # Choice was set by StartIni
        if self.Choice > 0:
            self.Buttons[self.Choice].select()
            # Do same as HandleDots.
            if (self.Choice != 4) and (self.Choice != 6):
                self.FillDirList()
                self.FillFileList()

    #------------------

    def Update(self):  # Share the wealth
        scene.ProcessSomeEvents(1)
        root.lift()
        root.after(150, self.Update)

    #------------------

    def ProtectedLoad(self,FullName):
        global SearchList
        OBJlist={} # use dict to avoid dups
        SearchList={}
        # Open the file and check the OBJ to see if it's present.
        try:
            fpProt=open(FullName,"rt")
        except:
            tkMessageBox.showwarning(title="File exists but can't be opened.",message=FullName)
            return 0
        while 1:
            st=fpProt.readline()
            if (st == None) or (st ==''):
                break
            elif string.count(st,'GetStringRes')or string.count(st,'Camera') or string.count(st,'spotLite'):
                continue  # These will always be findable
            elif string.count(st,'figureResFile'):
                # First get rid of newline and internal quotes if any
                st=string.strip(st)
                st=string.replace(st,'"','')
                st=string.replace(st,':','/') # De-mac
                Parts=string.split(st,' ',1)
                Remainder=Parts[1] # Remainder after figureResFile
                FullOBJ=os.path.normpath(BasePath+Remainder)
                OBJlist[FullOBJ]=1

            elif string.count(st,'objFileGeom'):
                st=string.strip(st)
                st=string.replace(st,'"','')
                st=string.replace(st,':','/') # De-mac
                Parts=string.split(st,' ',3) # to skip the 0 0 but leave rest together
                Remainder=Parts[3] # Remainder after objFileGeom 0 0
                FullOBJ=os.path.normpath(BasePath+Remainder)
                OBJlist[FullOBJ]=1
        fpProt.close()
        for FullOBJ in OBJlist.keys():
            # Try to find each where it belongs:
            if os.path.exists(FullOBJ): # Don't just quit here: Fig can have more than one OBJ
                continue
            else:
                CurDrive,dummy=os.path.splitdrive(FullOBJ)
                BaseOBJ=os.path.basename(FullOBJ)
                CurDrive=os.path.normpath(CurDrive+'/')
                os.path.walk(CurDrive,FindOBJ,BaseOBJ)
                if not SearchList: # Result of walk
                    tkMessageBox.showwarning(title="OBJ needed, can't be found:",message=BaseOBJ)
                    return 0
                else: # OBJ found somewhere
                    for OneOBJ in SearchList.keys():
                        T="%s in folder %s" % (OneOBJ, SearchList[OneOBJ])
                        AnswerYes=tkMessageBox.askyesno(title="OBJ found in wrong folder. Move OBJ and use?",message=T)
                        # If yes, copy the OBJ into the folder where it should be.
                        if AnswerYes:
                            SourcePath=SearchList[OneOBJ]+'/'+OneOBJ
                            DestPath=FullOBJ
                            shutil.copyfile(SourcePath,DestPath)
                        else:  # If no, cancel the whole thing.
                            return 0

        if string.count(string.upper(FullName),'.PZ3'):
            poser.CloseDocument(1) # Get rid of current
            poser.OpenDocument(FullName) # If everything in place, let the load happen.
        else:
            scene.LoadLibraryFigure(FullName) # If everything in place, let the load happen.
        return 1

    #------------------

    def SwapAll(self):  # called by mirror actions
        global BaseFig
        if not BaseFig: return
        scene.SetFrame(0)

        BaseFig.SwapTop()
        BaseFig.SwapBottom()
        MidlineList=["Neck","Head","Chest","Abdomen","Hip","Left Eye","Right Eye"]
        for Name in MidlineList:
            try:
                OneActor=BaseFig.Actor(Name)
            except:
                pass
            try:
                TwistParm=OneActor.ParameterByCode(poser.kParmCodeYROT)
                TwistParm.SetValue(-TwistParm.Value())
            except:
                pass
            try:
                SideParm=OneActor.ParameterByCode(poser.kParmCodeZROT)
                SideParm.SetValue(-SideParm.Value())
            except:
                pass

    #------------------

    def SwapFace(self,OneFile):  # called by face-mirror choice
        global BaseFig
        if not BaseFig: return
        try:
            OneActor=BaseFig.Actor("Head")
        except:
            return
        MTList={} # Uppercase MT names as keys, tuple (actual name, orig value) as element

        # Set up the list of all parameters with original values.
        for OneActor in BaseFig.Actors():
            AN = OneActor.Name()
            MTList[AN] = {} # subdict
            for OneParm in OneActor.Parameters():
                if OneParm.IsMorphTarget():
                    PN = OneParm.Name()
                    PV = OneParm.Value()
                    MTList[AN][string.strip(string.upper(PN))] = (PN,PV)

        # Now load the pose.
        scene.LoadLibraryFace(OneFile)

        # Try all four possible switchovers.  Ideally
        # this should be only L/R and LEFT/RIGHT, but
        # some commercial figures do have asymmetrical names.
        # Also, when we check for plain L, we look only
        # in first and last place.  Replacing L by R
        # everywhere leads to things like
        # DIMPLE L -> DIMPRE R .

        for AN in MTList.keys(): # Looking through the uppercase
            OneActor = BaseFig.Actor(AN)
            Subdict = MTList[AN]
            OppDict={}  # Keys are uppercase Left or L, vals are uppercase Right or R
            for N in Subdict.keys(): # Looking through the uppercase
                (RealN,oldV) = Subdict[N]
                if N[0]=='L':
                    OppName = 'R' + N[1:]
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName
                    OppName = string.replace(N,'L','RIGHT',1) # replace first L
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName

                elif N[-1]=='L':
                    OppName = N[:-1]+'R'
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName
                    OppName = N[:-1]+'RIGHT'
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName

                elif string.count(N,'LEFT'):
                    OppName = string.replace(N,'LEFT','R')
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName
                    OppName = string.replace(N,'LEFT','RIGHT')
                    if OppName in Subdict.keys():
                        OppDict[N]=OppName

            # Still at level of one actor, use the OppDict to swap.
            for UpperL in OppDict.keys():
                RealL,OldLval = Subdict[UpperL]
                UpperR = OppDict[UpperL]
                RealR,OldRval = Subdict[UpperR]
                NewLval = OneActor.Parameter(RealL).Value()
                NewRval = OneActor.Parameter(RealR).Value()
                if (NewLval != OldLval) or (NewRval != OldRval):
                    # Swap only the values that were changed by this pose.
                    OneActor.Parameter(RealR).SetValue(NewLval)
                    OneActor.Parameter(RealL).SetValue(NewRval)



    #------------------

    def SpawnMT(self,ObjFileName):
        TheFigure=0
        TheActor=scene.CurrentActor()  # There's always an actor.
        # Sort out types of Actors. Apply MTs only to Props and Body
        # Parts, not to Lights, etc
        if (not TheActor.IsProp()) and (not TheActor.IsBodyPart()):
            return
        if (TheActor.Name() == 'UNIVERSE') or (TheActor.Name() == 'GROUND'):
            return

        try:
            fp=open(ObjFileName,'rb')
        except:
            return 0

        # Get name
        MainName,ExtName=os.path.splitext(os.path.basename(ObjFileName))

        # Generate the dial, using the name
        TheActor.SpawnTarget(MainName)
        NewMTParm=TheActor.Parameter(MainName)

        # Set up Delta vars.
        DeltaX=0.0
        DeltaY=0.0
        DeltaZ=0.0

        OldGeom=TheActor.Geometry()
        NumVerts=OldGeom.NumVertices()

        # Read file. When we hit a line beginning with 'v ',
        # read the 3 values along with the corresponding
        # values in the OldGeom.  Put the deltas in the
        # new MT.
        Index=0
        LineParts=[]
        while Index < (NumVerts-1):
           try:
              st=fp.readline()
           except:
              break
           S=string.split(st)
           if not S: continue
           elif not S[0]: continue
           elif S[0] == 'v':  # This is a vertex line
               OneVert=OldGeom.Vertex(Index)
               DeltaX=float(S[1])-OneVert.X()
               DeltaY=float(S[2])-OneVert.Y()
               DeltaZ=float(S[3])-OneVert.Z()
               # Put deltas into MT
               NewMTParm.SetMorphTargetDelta(Index, DeltaX, DeltaY, DeltaZ)
               Index=Index+1  # Only inc when we append something
        fp.close() # Finished with file

    #------------------

    def StartIni(self):  # Called on startup
        global DefaultDirs
        global LatestDirs
        global PossDrives
        global StuffLabel

        LettersNoA='BCDEFGHIJKLMNOPQRSTUVWXYZ' # Look for all but A
        for i in LettersNoA:
            PathStr='%s:/' % i
            if os.path.exists(PathStr):
                PossDrives.append(PathStr)

        # See if we need to start fresh on INI.
        if not os.path.exists(IniName):
            # Write a fresh Ini file from Default Dirs
            fp=open(IniName,'w')
            # First write a default Choice value
            fp.write('0\n')
            # Then write the two Whatever labels on separate lines.
            st=StuffLabel+'\n'
            fp.write(st)
            # Now write the dirs:
            for OneString in DefaultDirs:
                st=BasePath+OneString+'\n'
                fp.write(st)
            fp.close() # Close from writing
        # Whether it's old or new, open for reading.
        fp=open(IniName,'r')
        LatestDirs=[]
        self.Choice=int(fp.readline())  # first item is latest Choice
        StuffLabel=string.rstrip(fp.readline())
        while 1: # Fill in the LatestDirs array from Ini.
            st=fp.readline()
            if not st: break
            stcut=string.rstrip(st)
            LatestDirs.append(stcut)
        fp.close()

    #------------------

    def WriteIni(self): # Called after action
        global StuffLabel
        try:
            fp=open(IniName,'w')
        except:
            tkMessageBox.showerror(IniName, 'Unable to write INI file')
            exit
        # First write the latest Choice number.
        fp.write(str(self.Choice)+'\n')
        # Then write the two Whatever labels on separate lines.
        # Pick up the last-placed contents.
        StuffLabel=self.What1Entry.get()
        st=StuffLabel+'\n'
        fp.write(st)
        # Then write from LatestDirs to the INI file.
        for OneString in LatestDirs:
            st='%s\n' % OneString
            fp.write(st)
        fp.close()

    #------------------

    def FillDirListMyComputer(self):
        global DirDisplayList
        global DirFullList
        # Special trick when the selected line is 'MyComputer'.  Because
        # the 'root' doesn't have any representation in the os.path
        # setup, we have to fake it.
        DirDisplayList=[]  # This will hold the short versions for display
        DirFullList=[]  # Parallel list of full paths so a click can immediately find...
        self.DirListBox.delete(0,END) # Clear the box
        DirDisplayList.append('My Computer') # Place the visible first line
        DirFullList.append('/')  # So that normpath will just eliminate it
        # Insert the PossDrives as the subdirs below current.
        for OneDir in PossDrives: # Reminder: this loop starts at index 1
            DirDisplayList.append(OneDir)
            DirFullList.append(OneDir)

        # Now go through and modify each entry in the Display list
        # to get the indentation right, and put the result immediately
        # into the actual listbox.
        for i in range(len(DirDisplayList)):
            OldStr=DirDisplayList[i]
            # In this case we know MyComputer (selected) is 0,
            # and each of the drives is indented equally.
            if i == 0:
                NewStr=OldStr
            else:
                NewStr=(' ')+OldStr
            # Fill the actual list with these indented strings.
            self.DirListBox.insert(END,NewStr)

        # Empty the file box.  No files are ever present in 'MyComputer'
        self.FileListBox.delete(0,END)

        # Finally, select MyComputer.
        self.DirListBox.selection_set(0)


    #------------------

    def FillDirList(self):
        global DirDisplayList
        global DirFullList
        DirDisplayList=[]  # This will hold the short versions for display
        DirFullList=[]  # Parallel list of full paths so a click can immediately find...
        self.DirListBox.delete(0,END) # Clear the box (important because
        # we re-use this routine each time a change is made.)
        CurrentDir=LatestDirs[self.Choice]
        CurDirNorm=os.path.normpath(CurrentDir)
        Rootmost,Rest=os.path.split(CurDirNorm)
        # Insert the current:
        DirDisplayList.insert(0,Rest)
        DirFullList.insert(0,CurDirNorm)
        # 1. Run down toward root, inserting each new entry on top.
        Rootmost=CurDirNorm
        for n in range(100):
             Rootmost,Rest=os.path.split(Rootmost)
             Dummy,ListItem=os.path.split(Rootmost)
             if not ListItem:
                 break
             DirDisplayList.insert(0,ListItem) # Insert on top.
             DirFullList.insert(0,Rootmost)
        # To get the drive, insert the Dummy part:
        Dummy=string.replace(Dummy,'\\\\','\\') # Get rid of extra slashes at this point
        DirDisplayList.insert(0,Dummy) # Insert on top.
        DirFullList.insert(0,Dummy)
        # Finally, put our fake 'My Computer' at the very top.
        DirDisplayList.insert(0,'My Computer') # Insert on top.
        DirFullList.insert(0,'My Computer') # Will not actually be used
        # from here, but keeps the indexes parallel.

        # At this point current dir is at bottom.
        # Get its index, which will remain the same from here on:
        IndexOfCurrent=len(DirDisplayList)-1

        # 2. Now find the childs of the current if any,
        # and insert them below current.
        ChildFind=CurDirNorm+'/*'
        ChildList=glob.glob(ChildFind)
        ChildList.sort(lccompare)

        for OneDir in ChildList:
            if os.path.isdir(OneDir):
                if OneDir != CurDirNorm:
                    Dummy,Rest=os.path.split(OneDir)
                    DirDisplayList.append(Rest)
                    DirFullList.append(OneDir)
        # Now go through and modify each entry in the Display list
        # to get the indentation right, and put the result immediately
        # into the actual listbox.
        for i in range(len(DirDisplayList)):
            OldStr=DirDisplayList[i]
            if i <= IndexOfCurrent: # without sort, was just <
                NewStr=(' ' * i)+OldStr
            else:
                NewStr=(' ' * (1+IndexOfCurrent))+OldStr
            # Fill the actual list with these indented strings.
            self.DirListBox.insert(END,NewStr)
        # Finally, select the current.
        self.DirListBox.selection_set(IndexOfCurrent)

    #------------------

    def HandleSelectDir(self,event):
        global LatestDirs
        Selecteds=event.widget.curselection() # Pick up the selection
        Index=int(Selecteds[0])  #Selecteds is tuple of indexes, but this box is SINGLE
        if Index == 0: # Special trick when selecting from 'MyComputer'.
            LatestDirs[self.Choice]='/' # slash will be dropped by normpath
            self.FillDirListMyComputer() # Reshape the Dir list box
            self.FillFileList()
        else:
            # Put the new selection (abspathed) into the Latestdirs list.
            LatestDirs[self.Choice]=DirFullList[Index]
            self.FillDirList() # Reshape the Dir list box
            self.FillFileList()

    #------------------

    def FillFileList(self):
        global FileList
        global LatestDirs
        global Patterns
        # Called by each of the dots that needs a file.
        # Get the filetype and dir from the Index.
        PresentDir=LatestDirs[self.Choice]
        PresentFileTypes=Patterns[self.Choice]
        # Split out the file types, which are written to
        # make splitting easy; then use glob to find all
        # files of these types in the indexed directory.
        FT=string.split(PresentFileTypes)
        self.FileListBox.delete(0,END) # Clear out whatever is there
        FileList =[] # Clear out whatever is there
        for OneType in FT:  # The types may include several extensions
           FullType=os.path.normpath(PresentDir+'/'+OneType) # note extra slash!
           ListPart=glob.glob(FullType)
           # This gives a list of full pathnames. Leave them as is for the
           # list of actual files, but carve them down for the listbox.
           for OneFullPath in ListPart:
                FileList.append(OneFullPath)
        FileList.sort(lccompare) # Alphabetize
        for OneFullPath in FileList:
            self.FileListBox.insert(END,os.path.basename(OneFullPath))

    #------------------
    # Handlers for the Load-select dots
    #------------------

    def HandleDots(self,event):
        self.Choice=int(event.widget.cget('value'))
        if (self.Choice != 4) and (self.Choice != 6):
            self.FillDirList()
            self.FillFileList()

    #------------

    def HandleUndo(self):
        # Restricted tightly. The Undo button is not even available
        # until you've done one of the Pose-like actions.  Those actions
        # call Memorize and configure the button to NORMAL.  All we do
        # here is call Reset for the appropriate item, and re-gray the button.
        if (self.Choice >= 3) and (self.Choice <= 8): # Pose types
            if not BaseFig: return
            BaseFig.Reset()
        elif self.Choice == 9 or self.Choice == 10: # Face types
            if not BaseFig: return
            FaceActor=BaseFig.Actor('Head')
            FaceActor.Reset()
        self.UndoButton.configure(state=DISABLED)

    #------------

    def HandleGo(self):
        # Main selector.  Happens when you hit go.
        global FileList
        global BaseFig
        # Pick up the quantity first, for use in the places where
        # it makes sense.
        try:
            self.Quantity=int(self.QuantityEntry.get())
        except:
            self.Quantity=1

        # Pick up the index and do the real action.
        Cursel=self.FileListBox.curselection()
        if not Cursel: return
        # Reminder: curselection gives a list of
        # strings like ['1','2','4'] not a list of integers.
        # Use the index from cursel to select the
        # corresponding entries in the actual FileList,
        # which contains the full absolute paths.
        SelectedFiles=[]
        for i in Cursel:
            SelectedFiles.append(FileList[int(i)])
        # Needed by some, so do it just once:
        # Get current fig
        BaseFig=0
        try:
            BaseFig=scene.CurrentFigure()
        except:
            pass
        TryParent=0
        try:
            TryParent=scene.CurrentActor()
        except:
            pass

        #-----
        # Long 'switch' statement for all indexes.
        #-----

        if  self.Choice == 1: # Figure Default
            for OneFile in SelectedFiles:
                for n in range(self.Quantity):
                    if not self.ProtectedLoad(OneFile):
                        # Returns 0 if error.  TKMessage already showed inside routine.
                        self.master.destroy()
                        self.master.quit()

        #-----

        elif self.Choice == 2: # Figure Conform
            for OneFile in SelectedFiles:
                for n in range(self.Quantity):
                    if not self.ProtectedLoad(OneFile):
                        # Returns 0 if error.  TKMessage already showed inside routine.
                        self.master.destroy()
                        self.master.quit()
                    # The new fig is now the current:
                    try:
                        NewFig=scene.CurrentFigure()
                    except:
                        NewFig=0
                    if BaseFig and NewFig: # If you selected a fig before this action:
                        NewFig.SetConformTarget(BaseFig)
                        scene.SelectFigure(BaseFig)
                    # Otherwise just do nothing.
                    # Return the CurrentFig to the base, so you can load
                    # other clothing in the same session.

        #-----

        elif (self.Choice == 3) or (self.Choice == 5) or (self.Choice == 7): # Pose types, Default
            # Memorize the pose before, so we can restore it.
            if not BaseFig: return
            BaseFig.Memorize()
            self.UndoButton.configure(state=NORMAL)
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryPose(OneFile) # Load the pose.
                except:
                    pass

        #-----

        elif (self.Choice == 4) or (self.Choice == 6) or (self.Choice == 8): # Pose types, mirrored
            if not BaseFig: return
            BaseFig.Memorize()
            self.UndoButton.configure(state=NORMAL)
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryPose(OneFile) # Load the pose.
                    self.SwapAll()
                except:
                    pass

        #-----

        elif self.Choice == 9: # Face Default
            # Memorize the face before, so we can restore it.
            if not BaseFig: return
            try:
                FaceActor=BaseFig.Actor('Head')
            except:
                return
            FaceActor.Memorize()
            self.UndoButton.configure(state=NORMAL)
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryFace(OneFile) # Load the face.
                except:
                    pass

        #-----

        elif self.Choice == 10: # Face mirrored
            if not BaseFig: return
            try:
                FaceActor=BaseFig.Actor('Head')
            except:
                return
            FaceActor.Memorize()
            self.UndoButton.configure(state=NORMAL)
            for OneFile in SelectedFiles:
                self.SwapFace(OneFile)

        #-----

        elif self.Choice == 11: # Hair Default
            # Hair can be HR2, PP2, CR2
            for OneFile in SelectedFiles:
                Main,Ext=os.path.splitext(OneFile)
                Ext=string.upper(Ext)
                try:
                    if string.count(Ext,'CR'):
                        if not self.ProtectedLoad(OneFile):
                            # Returns 0 if error.  TKMessage already showed inside routine.
                            self.master.destroy()
                            self.master.quit()
                    elif string.count(Ext,'PP'):
                        scene.LoadLibraryProp(OneFile) # Load one prop
                    elif string.count(Ext,'HR'):
                        scene.LoadLibraryHair(OneFile) # Load one hair
                except:
                    pass

        #-----

        elif self.Choice == 12: # Hair Parented
            # Pick up the figure's head; arrange so that
            # we still load the hair if we can't find
            # a current figure with a head.
            HeadActor=0
            if BaseFig:
                try:
                    HeadActor=BaseFig.Actor('Head')
                except:
                    pass
            for OneFile in SelectedFiles:
                Main,Ext=os.path.splitext(OneFile)
                Ext=string.upper(Ext)
                try:
                    if string.count(Ext,'CR'):
                        if not self.ProtectedLoad(OneFile):
                            # Returns 0 if error.  TKMessage already showed inside routine.
                            self.master.destroy()
                            self.master.quit()
                    elif string.count(Ext,'PP'):
                        scene.LoadLibraryProp(OneFile) # Load one prop
                    elif string.count(Ext,'HR'):
                        scene.LoadLibraryHair(OneFile) # Load one hair
                except:
                    pass

                # For CR2, conform; for others, parent.
                if string.count(Ext,'CR'): # Hair is a Figure; handle like clothing.
                    try:
                        NewFig=scene.CurrentFigure() # The new fig is now the current
                    except:
                        NewFig=0
                    if BaseFig and NewFig: # If you selected a fig before this action:
                        NewFig.SetConformTarget(BaseFig)
                        # Return the CurrentFig to the base, so you can load
                        # other clothing in the same session.
                        scene.SelectFigure(BaseFig)
                else: # Other than CR2, parent to Head
                    try:
                        NewActor=scene.CurrentActor() # The new is Current
                        NewActor.SetParent(HeadActor) # Parent to head
                    except:
                        pass

        #-----
        elif self.Choice == 13: # Left Hand
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryHand(OneFile,0) # Load left hand
                except:
                    pass
        #-----
        elif self.Choice == 14: # Right Hand
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryHand(OneFile,1) # Load right hand
                except:
                    pass

        #-----
        elif self.Choice == 15: # Prop Default
            for OneFile in SelectedFiles:
                for n in range(self.Quantity):
                    try:
                        scene.LoadLibraryProp(OneFile) # Load one prop
                    except:
                        pass
        #-----
        elif self.Choice == 16: # Prop Parented
            # Get the current actor first.
            for OneFile in SelectedFiles:
                for n in range(self.Quantity): # Now load the prop N times.
                    try:
                        scene.LoadLibraryProp(OneFile) # Load prop
                    except:
                        pass
                    if TryParent:
                        NewProp=scene.CurrentActor()
                        NewProp.SetParent(TryParent)
        #-----
        elif self.Choice == 17: # Lights Default
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryLight(OneFile) # Load light
                except:
                    pass
        #-----
        elif self.Choice == 18: # Lights Parented
            # Get the current actor first.
            TryParent=0
            try:
                TryParent=scene.CurrentActor()
            except:
                pass
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryLight(OneFile) # Load light set
                except:
                    pass
                # If we have a current actor, parent all lights to that actor.
                if TryParent:
                    for OneLight in scene.Lights():
                        OneLight.SetParent(TryParent)
        #-----
        elif self.Choice == 19: # Cameras Default
            for OneFile in SelectedFiles:
                try:
                    scene.LoadLibraryCamera(OneFile) # Load camera set
                except:
                    pass
        #-----
        elif self.Choice == 20: # Cameras Parented
            for OneFile in SelectedFiles:
                # Get the current actor first.
                try:
                    scene.LoadLibraryCamera(OneFile) # Load camera set
                except:
                    pass
                # If we have a current actor, parent all lights to that actor.
                if TryParent:
                    for OneCamera in scene.Cameras():
                        OneCamera.SetParent(TryParent)
        #-----
        elif self.Choice == 21: # Import Mesh
            imex=scene.ImExporter()
            for OneFile in SelectedFiles:
                Main,Ext=os.path.splitext(OneFile)
                Ext=string.upper(Ext)
                if string.count(Ext,'OBJ'):
                    imoptions=imex.ImportOptions('obj', None)
                    imoptions[poser.kImOptCodeAUTOSCALE]=0
                    for n in range(self.Quantity):
                        imex.Import('OBJ', 'File Format OBJ', OneFile, imoptions)
                elif string.count(Ext,'3DS'):
                    imoptions=imex.ImportOptions('3DS', None)
                    imoptions[poser.kImOptCodeAUTOSCALE]=0
                    for n in range(self.Quantity):
                        imex.Import('3DS', 'File Format 3DS', OneFile, imoptions)
                elif string.count(Ext,'BVH'):
                    imoptions=imex.ImportOptions('BVH', None)
                    imoptions[poser.kImOptCodeAUTOSCALE]=0
                    for n in range(self.Quantity):
                        imex.Import('BVH',None, OneFile)
                elif string.count(Ext,'DXF'):
                    imoptions=imex.ImportOptions('DXF', None)
                    imoptions[poser.kImOptCodeAUTOSCALE]=0
                    for n in range(self.Quantity):
                        imex.Import('DXF', 'File Format DXF', OneFile, imoptions)
                elif string.count(Ext,'3DM'): # 3DM or 3DMF
                    imoptions=imex.ImportOptions('3DMF', None)
                    imoptions[poser.kImOptCodeAUTOSCALE]=0
                    for n in range(self.Quantity):
                        imex.Import('3DMF', 'File Format 3DMF', OneFile, imoptions)

        #-----
        elif self.Choice == 22: # ImportMorph
            for OneFile in SelectedFiles:
                self.SpawnMT(OneFile)

        #-----
        elif self.Choice == 23: # Stuff default
            for OneFile in SelectedFiles:
                Main,Ext=os.path.splitext(OneFile)
                Ext=string.upper(Ext)
                try:
                    if string.count(Ext,'CR') or string.count(Ext,'PZ'):
                        if not self.ProtectedLoad(OneFile):
                            # Returns 0 if error.  TKMessage already showed inside routine.
                            self.master.destroy()
                            self.master.quit()
                    elif string.count(Ext,'PP'):
                        scene.LoadLibraryProp(OneFile) # Load one prop
                    elif string.count(Ext,'Z2') or string.count(Ext,'2Z'):
                        scene.LoadLibraryPose(OneFile) # Load one pose
                    elif string.count(Ext,'HR'):
                        scene.LoadLibraryHair(OneFile) # Load one hair
                    elif string.count(Ext,'FC'):
                        scene.LoadLibraryFace(OneFile) # Load one face
                    elif string.count(Ext,'HD'):
                        scene.LoadLibraryHand(OneFile,0) # Load one hand, arb. left
                    elif string.count(Ext,'LT'):
                        scene.LoadLibraryLight(OneFile) # Load one light
                    elif string.count(Ext,'CM'):
                        scene.LoadLibraryCamera(OneFile) # Load one cam
                except:
                    pass

        #-----

        elif self.Choice == 24: # Stuff parented
            for OneFile in SelectedFiles:
                Main,Ext=os.path.splitext(OneFile)
                Ext=string.upper(Ext)
                try:
                    if string.count(Ext,'CR'):
                        if not self.ProtectedLoad(OneFile):
                            # Returns 0 if error.  TKMessage already showed inside routine.
                            self.master.destroy()
                            self.master.quit()
                    elif string.count(Ext,'PP'):
                        scene.LoadLibraryProp(OneFile) # Load one prop
                    elif string.count(Ext,'Z2') or string.count(Ext,'2Z'):
                        scene.LoadLibraryPose(OneFile) # Load one pose
                    elif string.count(Ext,'HR'):
                        scene.LoadLibraryHair(OneFile) # Load one hair
                    elif string.count(Ext,'FC'):
                        scene.LoadLibraryFace(OneFile) # Load one face
                    elif string.count(Ext,'HD'):
                        scene.LoadLibraryHand(OneFile,0) # Load one hand, arb. left
                    elif string.count(Ext,'LT'):
                        scene.LoadLibraryLight(OneFile) # Load one light
                    elif string.count(Ext,'CM'):
                        scene.LoadLibraryCamera(OneFile) # Load one cam
                except:
                    pass

                # For CR2, conform; for others, parent.
                if string.count(Ext,'CR'): # Shoe is a Figure; handle like clothing.
                    try:
                        NewFig=scene.CurrentFigure() # The new fig is now the current
                    except:
                        NewFig=0
                    if BaseFig and NewFig: # If you selected a fig before this action:
                        NewFig.SetConformTarget(BaseFig)
                        # Return the CurrentFig to the base, so you can load
                        # other clothing in the same session.
                        scene.SelectFigure(BaseFig)
                else: # Other than CR2, parent.
                    try:
                        NewActor=scene.CurrentActor() # The new is Current
                        NewActor.SetParent(TryParent) # Parent to original current
                    except:
                        pass

        #-----end switch statement

        self.FileListBox.selection_clear(0,END) # Clear the selections in the file box
        self.FileTypeVar.set(0) # Clear the dots
        self.Quantity=1  # Restore the quantity to 1
        self.QuantityEntry.delete(0,'end')
        self.QuantityEntry.insert(0,'1')
        scene.DrawAll() # force redraw of scene
        self.WriteIni() # Refresh the INI file.
        self.GoButton.update_idletasks() # Make the button pop
    # end of long HandleGo routine

    #------------------

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root=Tk()
app=App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.

#-----------------------------------------------------------------------------
