#------------------------------------------------------------
"""

 SwapBodyMorphs.py

    Flip the symmetry of MTs on the body.

    This does two separate things:

    1. On centerline parts [Neck,Chest,Abdomen,Hip] it switches any MTs
    with left/right names.  So, if Neck has Goiter_L and Goiter_R
    morphs, their values will be swapped.

    2. On paired parts [Left Collar / Right Collar, etc.] it switches
    MTs with the same name.  So if Left Shoulder has a SuperNerd morph,
    and Right Shoulder also has a SuperNerd morph, the values of those
    two will be switched.

    Note on naming:  We try all four possible switchovers.  Ideally this
    should be only L/R and LEFT/RIGHT, but some commercial figures do
    have asymmetrical names, like "Brow-Angry-Right" and "Brow-Angry-L"
    on Koshini.

 Copyright (c) 2003 David G. Drumright (ockham)

"""

#------------------------------------------------------------

import poser
import string
scene = poser.Scene()

#-----------------------------------------------------------------------------
BaseFig = scene.CurrentFigure() # should have been set
if not BaseFig: exit

# Begin centerline loop
CenterlinePartsUC = ["BODY","NECK","CHEST","ABDOMEN","HIP"]
for OneBodyPart in BaseFig.Actors():
    OneBodyPartUC = string.strip(string.upper(OneBodyPart.Name()))
    if OneBodyPartUC not in CenterlinePartsUC: continue

    # Look for MTs that have same name except for Right/Left
    MTList={} # Uppercase MT names as keys, actual names as values.
    for OneParm in OneBodyPart.Parameters():
        if OneParm.IsMorphTarget():
            N = OneParm.Name()
            MTList[string.upper(N)] = N

    OppDict={}  # Keys are uppercase Left or L, vals are uppercase Right or R


    for N in MTList.keys():
        if N[0]=='L':
            OppName = 'R' + N[1:]
            if OppName in MTList.keys():
                OppDict[N]=OppName
            OppName = string.replace(N,'L','RIGHT',1) # replace first L
            if OppName in MTList.keys():
                OppDict[N]=OppName

        elif N[-1]=='L':
            OppName = N[:-1]+'R'
            if OppName in MTList.keys():
                OppDict[N]=OppName
            OppName = N[:-1]+'RIGHT'
            if OppName in MTList.keys():
                OppDict[N]=OppName

        elif string.count(N,'LEFT'):
            OppName = string.replace(N,'LEFT','R')
            if OppName in MTList.keys():
                OppDict[N]=OppName
            OppName = string.replace(N,'LEFT','RIGHT')
            if OppName in MTList.keys():
                OppDict[N]=OppName

    # Now use the OppDict to swap.
    for UpperL in OppDict.keys():
        RealL = MTList[UpperL]
        UpperR = OppDict[UpperL]
        RealR = MTList[UpperR]
        Lval = OneActor.Parameter(RealL).Value()
        Rval = OneActor.Parameter(RealR).Value()
        OneActor.Parameter(RealR).SetValue(Lval)
        OneActor.Parameter(RealL).SetValue(Rval)

#------------------------------------------------------------

# Begin side-parts loop.  In this case we do the L/R switch
# on body parts and use same-name parameters.

PartList={} # Uppercase part names as keys, actual part names as values.
for OneBodyPart in BaseFig.Actors():
    OBPLC = string.strip(OneBodyPart.Name())
    OBPUC = string.upper(OBPLC)
    if OBPUC not in CenterlinePartsUC:
        PartList[OBPUC]=OBPLC

# Run through this list and create the OppDict:
OppDict={}  # Keys are uppercase Left or L, vals are uppercase Right or R

for N in PartList.keys():
    if string.count(N,'LEFT'):
        OppName = string.replace(N,'LEFT','R')
        if OppName in PartList.keys():
            OppDict[N]=OppName
        OppName = string.replace(N,'LEFT','RIGHT')
        if OppName in PartList.keys():
            OppDict[N]=OppName
    elif N[0]=='L':
        OppName = 'R' + N[1:]
        if OppName in PartList.keys():
            OppDict[N]=OppName
        OppName = string.replace(N,'L','RIGHT',1) # replace first L
        if OppName in PartList.keys():
            OppDict[N]=OppName

    elif N[-1]=='L':
        OppName = N[:-1]+'R'
        if OppName in PartList.keys():
            OppDict[N]=OppName
        OppName = N[:-1]+'RIGHT'
        if OppName in PartList.keys():
            OppDict[N]=OppName


# Now go through the OppDict. In this case, we look at all
# the MTs on each paired part; when we find a match, which
# we should on a well-made model, we swap.
for UpperL in OppDict.keys():
    RealL = PartList[UpperL]
    UpperR = OppDict[UpperL]
    RealR = PartList[UpperR]
    LMTs=[]
    RMTs=[]
    # Make lists of MTs in each, for convenience.
    LActor = BaseFig.Actor(RealL)
    RActor = BaseFig.Actor(RealR)
    for OneParm in LActor.Parameters():
        if OneParm.IsMorphTarget():
            LMTs.append(OneParm.Name())
    for OneParm in RActor.Parameters():
        if OneParm.IsMorphTarget():
            RMTs.append(OneParm.Name())
    # Now look for matches.
    for OneName in LMTs:
        if OneName in RMTs:
            Lval = LActor.Parameter(OneName).Value()
            Rval = RActor.Parameter(OneName).Value()
            RActor.Parameter(OneName).SetValue(Lval)
            LActor.Parameter(OneName).SetValue(Rval)

#------------------------------------------------------------
