#------------------------------------------------------------
"""

 SwapFaceMorphs.py

    Flip the symmetry of MTs on the Head.

    Switches any MTs with left/right names.  So, if Head has Smile_L and
    Smile_R morphs, or LeftFrown and RightFrown, their values will be
    swapped.

    Note on naming:  We try all four possible switchovers.  Ideally this
    should be only L/R and LEFT/RIGHT, but some commercial figures do
    have asymmetrical names, like "Brow-Angry-Right" and "Brow-Angry-L"
    on Koshini.

 Copyright (c) 2003 David G. Drumright (ockham)

"""

#------------------------------------------------------------

import poser
import string
scene = poser.Scene()

#-----------------------------------------------------------------------------
BaseFig = scene.CurrentFigure() # should have been set
if not BaseFig: exit

try:
    OneBodyPart = BaseFig.Actor("Head")
except:
    raise "No head!"

# Look for MTs that have same name except for Right/Left
MTList={} # Uppercase MT names as keys, actual names as values.
for OneParm in OneBodyPart.Parameters():
    if OneParm.IsMorphTarget():
        N = OneParm.Name()
        MTList[string.upper(N)] = N

OppDict={}  # Keys are uppercase Left or L, vals are uppercase Right or R


for N in MTList.keys():
    if string.count(N,'LEFT'):
        OppName = string.replace(N,'LEFT','R')
        if OppName in MTList.keys():
            OppDict[N]=OppName
        OppName = string.replace(N,'LEFT','RIGHT')
        if OppName in MTList.keys():
            OppDict[N]=OppName
    elif N[0]=='L':
        OppName = 'R' + N[1:]
        if OppName in MTList.keys():
            OppDict[N]=OppName
        OppName = string.replace(N,'L','RIGHT',1) # replace first L
        if OppName in MTList.keys():
            OppDict[N]=OppName

    elif N[-1]=='L':
        OppName = N[:-1]+'R'
        if OppName in MTList.keys():
            OppDict[N]=OppName
        OppName = N[:-1]+'RIGHT'
        if OppName in MTList.keys():
            OppDict[N]=OppName


    # Now use the OppDict to swap.
    for UpperL in OppDict.keys():
        RealL = MTList[UpperL]
        UpperR = OppDict[UpperL]
        RealR = MTList[UpperR]
        Lval = OneActor.Parameter(RealL).Value()
        Rval = OneActor.Parameter(RealR).Value()
        OneActor.Parameter(RealR).SetValue(Lval)
        OneActor.Parameter(RealL).SetValue(Rval)

#------------------------------------------------------------
